		;; adding tasks to the system

#include "system.h"
		
.global addtask

		;; addtask
		;; < X/Y = address of code to execute
		;; < A = priority (1-7)
		
		;; (no error check)

		;; changes:		tmpzp:	0 1 2 3 4

addtask:
		;; find+allocate slot in taskqueue
		php
		sei						; no IRQ in here
		stx  tmpzp+3
		sty  tmpzp+4			; remember code address
		sta  tmpzp
		ldx  #31
	-	lda  lk_tstatus,x
		beq  +					; found a unused slot
		dex
		bpl  -
		bmi  skipadd				; (always jump)
		
		;; allocate slot

	+	ldy  lk_ipid
		bmi  +
		lda  lk_tstatus,y
	+	and  #tstatus_pri		; get priority of current task
		cmp  tmpzp
		bcc  +
		lda  tmpzp				; only use wanted priority, if it is
		                        ; smaller, than that of the current task
		bne  +
		lda  #1					; pri must be at least 1!
		
	+	ora  #tstatus_susp|tstatus_nosig|tstatus_szu
		sta  lk_tstatus,x		; flags:  susp, nosig, szu

		stx  tmpzp+2			; remember IPID of new task

		;; do as less as possible for the new task (he should do as much as
		;; possible on itself!)
		
		;; set up TSP  (task-super-page)

		ldy  #$80				; no I/O (since it is accessed from the kernel,
		jsr  spalloc			; which runs with I/O enabled)
		bcc  +					; got a page, so continue
		
		ldx  tmpzp+2
		lda  #0					; release task slot
		sta  lk_tstatus,x		
skipadd:
		plp
		sec						; error (try again later)
		rts

	+	stx  tmpzp+1			; address of allocated page
		ldy  #0
		sty  tmpzp

		;; set up simple stack
		ldy  #255
		lda  #>task_init		; address hi-byte (init)
		sta  (tmpzp),y			; (1)
		dey
		lda  #<task_init		; address lo-byte (init)
		sta  (tmpzp),y			; (2)
		dey
		lda  #0					; SR=0
		sta  (tmpzp),y			; (3)
		dey
		sta  (tmpzp),y			; A=0 (4)
		dey
		sta  (tmpzp),y			; X=0 (5)
		dey
		sta  (tmpzp),y			; Y=0 (6)
		dey
		lda  1					; use same memory configuration
		sta  (tmpzp),y			; memconfig=standard (7 bytes)

		lda  #7					; stacksize=7
		ldy  #tsp_stsize
		sta  (tmpzp),y

		lda  lk_taskcnt			; increas task counter and set PID of task
		adc  #1
		sta  lk_taskcnt
		ldy  #tsp_pid
		sta  (tmpzp),y
		lda  lk_taskcnt+1
		adc  #0
		sta  lk_taskcnt+1
		iny
		sta  (tmpzp),y

		lda  lk_ipid			; set ippid
		ldy  #tsp_ippid			; (internal parent process ID)
		sta  (tmpzp),y

		lda  tmpzp+3			; address lo-byte (task code)
		ldy  #tsp_syszp
		sta  (tmpzp),y			; put it to syszp
		lda  tmpzp+4			; address hi-byte (task code)
		iny
		sta  (tmpzp),y			; to syszp+1

		ldx  tmpzp+2			; IPID of new task
		lda  tmpzp+1
		sta  lk_ttsp,x			; set pointer to tsp

		jsr  p_insert			; insert into task CPU-queue

		plp
		clc
		rts						; return without error

		;; this is what the new task will execute first
task_init:
		lda  #>(suicide-1)
		pha
		lda  #<(suicide-1)
		pha						; last rts in the task will jump to "suicide"
		lda  syszp+1
		pha						; address (hi) of task's code
		lda  syszp
		pha						; address (lo) of task's code (need rti!!)
		php

		;; set up tsp

		lda  #0
		ldy  #tsp_zpsize
	-	sta  (lk_tsp),y			; initialize time,wait,semmap,zpsize
		dey						; with $00
		bpl  -

		ldy  #tsp_signal_vec	; initialize signal handler vector table
	-	sta  (lk_tsp),y			; (no installed signal handler)
		iny
		cpy  #tsp_signal_vec+16
		beq  -
		
		ldx  lk_ipid
		lda  lk_tstatus,x
		and  #$ff-(tstatus_szu|tstatus_nosig)
		sta  lk_tstatus,x
		
		rti						; jump to task's code
