		;; error and termination

#include "system.h"
#include "kerrors.h"

.global catcherr
.global suicerrout
.global print_error
.global suicide
.global hexout
		
		;; A=error code, that should be returned to main program
catcherr:
		sec
		rts

		;; kill current task, but print a error message first
		;; A=error code
suicerrout:
		jsr  print_error

		;; kill current task
		;; A=exit code
suicide:
		;; unlock used ressources
		pha
		sei
		ldx  lk_ipid
		lda  lk_tstatus,x
		ora  #tstatus_szu|tstatus_nosig
		sta  lk_tstatus,x		; used system zeropage (syszp)
		cli
		lda  #0
		sta  lk_locktsw			; be sure IRQ+taskswitches are enabled

		pla
		sta  syszp				; syszp=exitcode
		lda  #4
		sta  syszp+3
		lda  #32
		sta  syszp+2

	-	clc
		lda  syszp+3
		adc  #tsp_semmap
		tay
		lda  (lk_tsp),y
		beq  ++					; skip 8 semaphores
		ldx  syszp+2
	-	lsr  a
		bcc  +
		;; need unlock
		jsr  unlock
		jmp  --
	+	inx
		bne  -
	+	dec syszp+3
		bpl  --

		;; enable nmi, if it had been disabled
		ldx  lk_ipid
		lda  lk_tstatus,x
		and  #tstatus_nonmi
		beq  +
		jsr  enable_nmi

		;; free all used internal memory (exept tsp)
	+	lda  lk_ipid
		ldx  #2					; first possible page
	-	cpx  lk_tsp+1
		beq  +
		cmp  lk_memown,x
		bne  +
		txa
		pha
		jsr  pfree
		pla
		tax
	+	inx
		bne  -

		;; look for children, that try to serve their exitcode to this
		;; task

		lda  #waitc_zombie
		ldx  lk_ipid
		jsr  mun_block			; unblock all affected tasks
		
		;; serve exitcode
		sei
		ldy  #tsp_ippid
		lda  (lk_tsp),y			; get IPID of parent process
		bmi  exdone					; skip if there is no parent

		tax						; check if, parent is already waiting
		lda  lk_tstatus,x
		and  #tstatus_susp|tstatus_nosig ; is parent suspended or killed ?
		beq  +					; not, then ok
		and  #tstatus_susp
		beq  exdone				; skip if parent is already dieing
		lda  lk_ttsp,x
		sta  syszp+3
		lda  #0
		sta  syszp+2
		ldy  #tsp_wait0
		lda  (syszp+2),y
		cmp  #waitc_wait		; suspended because of 'wait'
		bne  +					; not, then ok

		;; parent is already waiting for my exitcode
		jsr  p_insert			; wakeup parent
		jmp  ++
		
		;; parent does something else, try to send signal sig_chld
	+	lda  #sig_chld
		sta  tmpzp+4
		jsr  _raw_sendsignal
		
	+	lda  syszp
		ldy  lk_ipid
		sta  lk_tslice,y		; current_task->tslice=exitcode
		lda  #waitc_zombie		; so he can fetch my exitcode (X=PPID)
		jsr  block				; suspend current task
		;; (continues, if parent read the exitcode)
exdone:	cli
		
		;; done...
		sei
		ldx  lk_tsp+1
		jsr  pfree				; free tsp
		
		ldx  lk_ipid			; get ipid of next task
		jsr  p_remove			; remove task from CPU-queue

		lda  #0
		sta  lk_locktsw			; make sure there will be a taskswitch
		sta  lk_tstatus,x		; remove from task list
		ldy  lk_tnextt,x		; IPID of next task
		lda  #$ff
		sta  lk_tnextt,x		; destroy old next pointer
		tya
		bmi  +
		ora  #$80
		sta  lk_ipid			; switch without saving current context

		jsr  force_taskswitch	; switch to next task

		;; nowhere to switch to, then jump to idle_task
	+	sta  lk_ipid			; $ff -> IPID (system is IDLE)
		cli
		jmp  idle_task
		
		;; A=error code
print_error:
		pha
		ldx  #0
	-	lda  err_txt,x
		beq  +
		jsr  printk
		inx
		bne  -
	+	pla
		pha
		jsr  hexout
		lda  #$0a
		jsr  printk
		pla
		rts

hexout:	pha
		lsr  a
		lsr  a
		lsr  a
		lsr  a
		tax
		lda  hextab,x
		jsr  printk
		pla
		and  #15
		tax
		lda  hextab,x
		jmp  printk
		
hextab:	.asc "0123456789abcdef"		
		
err_txt:
		.asc "CAUGHT ERROR #"
		.byte $00