		;; main initialization

#include "config.h"
		
#include MACHINE_H		
#include "system.h"
#include "debug.h"

main_init:
		sei						; disable interrupts
		cld						; clear decimal flag
		ldx  #255
		txs						; init stack pointer

#		include MACHINE(reset.s)
	
		;; set pointer to new interrupt routine
		ldx  #<irq_handler
		ldy  #>irq_handler
		stx  $fffe
		sty  $ffff
		ldx  #<nmi_handler
		ldy  #>nmi_handler
		stx  $fffa
		sty  $fffb
		ldx  #<panic			; reset_handler ???
		ldy  #>panic			; never called, because ROM will be there
		stx  $fffc
		sty  $fffd

		;; first init process table
		lda  #0
		ldx  #$20				; max number of tasks (32)
	-	sta  lk_tstatus,x
		sta  lk_tnextt,x
		sta  lk_tslice,x
		sta  lk_ttsp,x
		dex
		bpl  -

		;; 2nd init system data
		lda  #$ff				; "system is idle" and there is no task
		sta  lk_ipid			; to switch to
		lda  #1
		sta  lk_timer			; switch to next task on next IRQ
		lda  #0
		sta  lk_tsp				; (just empty)
		sta  lk_systic			; reset system's jiffy-counter
		sta  lk_systic+1
		sta  lk_systic+2
		sta  lk_sleepcnt
		sta  lk_sleepcnt+1		; wait a long time before alling _wakeup
	    lda  #0
		sta  lk_locktsw			; taskswitching is enabled (default)
		sta  lk_sleepipid		; no one to wakeup (no task is sleeping)
		lda  #0
		sta  lk_semmap
		sta  lk_semmap+1
		sta  lk_semmap+2
		sta  lk_semmap+3
		sta  lk_semmap+4		; no ressources used
		sta  lk_taskcnt
		sta  lk_taskcnt+1		; no tasks jet
		sta  lk_nmidiscnt

		;; initialize internal memory
		ldx  #0
	-	lda  #0
		sta  lk_memnxt,x
		lda  #$ff
		sta  lk_memown,x
		inx
		bne  -
		
		ldx  #31
	-	lda  _initmemmap,x
		sta  lk_memmap,x
		dex
		bpl  -
		
		;; clean up zeropage
		ldx  #$0f				; 16 bytes for each segment
		lda  #0
	-	sta  tmpzp,x
		sta  syszp,x
		sta  userzp,x
		dex
		bpl  -

#		include MACHINE(irqinit.s)
				
		jsr  locktsw			; disable taskswitches with enabled IRQ
		cli						; taskswitching is still disabled (!)
		
		;; init console and print short welcome message
		jsr  console_init
		ldx  #0
	-	lda  welcome_txt,x
		beq  +
		jsr  printk
		inx
		bne  -

		;; calibrate delay loop
	+	jsr  calibrate_delay

		ldx  #<my_task
		ldy  #>my_task
		jsr  addtask			; add (first) task

		jsr  unlocktsw
		
	-	jmp  -					; fade away (should not be reached)
		
welcome_txt:
		.asc "wELCOME TO LUNIX (lng) v0.07"
		.byte $0a,$00

		;; initial memory map (every 1-bit is an available page)

#		include MACHINE(initmemmap.s)

		;; **************************************************************
		;;  first task
		;; **************************************************************
		
my_task:
		ldx  lk_ipid
		lda  lk_tstatus,x
		ora  #7
		sta  lk_tstatus,x

		ldx  #0
	-	lda  hello_txt,x
		beq  +
		jsr  printk
		inx
		bne  -
		
		;; install keyboard scanner
	+	jsr  keyboard_init

		;; test
	-	ldx  #<32				; 32 jiffies = 1/2 s second
		ldy  #>32
		jsr  sleep

ploop:
		jsr  keyb_getc
		beq  -
		cmp  #"."
		bne  +
		
		;; add a child
		ldx  #<my_2task
		ldy  #>my_2task
		lda  #1					; lowest pri possible
		jsr  addtask
		jmp  ploop
		
	+	cmp  #","
		bne  +

		;;
		ldy  #0
		ldx  #0
	-	inc  debug2
		inx
		bne  -
		iny
		bne  -
		jmp  ploop
		
	+	cmp  #1
		bne  +

		;;
		jsr  micro_term
		jmp  ploop
		
	+	jsr  printk

		clc						; non blocking
		ldx  #<wait_struct
		ldy  #>wait_struct
		jsr  wait				; look for terminated child
		bcs  ploop

		pha
		ldx  #0
	-	lda  child_message_txt,x
		beq  +
		jsr  printk
		inx
		bne  -
	+	pla
		jsr  hexout
		lda  #" "
		jsr  printk
		ldy  #0
	-	lda  wait_struct,y
		jsr  hexout
		iny
		cpy  #7
		bne  -
		lda  #$0a
		jsr  printk
		
		jmp  ploop

hello_txt:
		.asc "wOw !  i'M ALIVE !"
		.byte $0a,$00

wait_struct:	.buf 7			; 7 bytes

child_message_txt:
		.byte $0a
		.asc "cHILD TERMINATED : \0"

		
		;; **************************************************************
		;;  second task(s)
		;; **************************************************************
		
my_2task:
		lda  #2
		jsr  set_zpsize			; use 1 byte from userzp
		lda  #"9"
		sta  userzp

	-	lda  userzp
		cmp  #"0"-1
		beq  ++
		cmp  #"5"
		bne  +
		ldy  #tsp_pid
		sei
		lda  (lk_tsp),y
		ldx  debug1
		cli
		ldy  #0
		sty  userzp+1
	-	sei
		sta  debug1
		stx  debug1
		sta  debug1
		stx  debug1
		sta  debug1
		stx  debug1
		sta  debug1
		stx  debug1
		cli
		iny
		bne  -
		inc  userzp+1
		bne  -
		lda  #"5"
	+	jsr  printk
		ldx  #<64
		ldy  #>64
		jsr  sleep
		dec  userzp
		bne  --

	+	ldy  #tsp_pid
		lda  (lk_tsp),y
		rts
