		;; micro terminal for LNG
		;; using swiftlink (over NMI)

#include "system.h"
#include "debug.h"
		
		swift_base equ $de00			; base address of swiftlink

		swift_io      equ swift_base+0
		swift_status  equ swift_base+1
		swift_command equ swift_base+2
		swift_control equ swift_base+3

.global micro_term
		
nmi_struct:
		jmp  nmi_handler
		jmp  nmi_disable
		jmp  nmi_enable
		
rs232_init:
		lda  #0
		sta  r_wrpointer		; write pointer into receive buffer
		sta  r_rdpointer		; write pointer into receive buffer
		sta  smode				; $80 if receive buffer if full
		;; (check if swiftlink is already in use ?)
		sta  swift_status		; reset of swiftlink
		lda  swift_status		; clear interrupt flag
		lda  swift_io			; clear rx-interrupt
		
		ldx  #<nmi_struct
		ldy  #>nmi_struct
		jmp  hook_nmi			; hook into system and call nmi_enable
								; (as soon as possible)
		
		
nmi_enable:
		lda  smode
		ora  #$40
		sta  smode
		bmi  +					; skip if receive buffer is already full
		lda  #%00011100			; 1 stopbit, 8 databits, 9600 baud (internal)
		sta  swift_control
		lda  #5
		sta  debug1
		lda  #%00001001			; no parity, no echo, no XMIT-IRQ,
								;  RECV-IRQ enabled, RTS on, DTR low
		sta  swift_command
	+	plp
		rts
		
nmi_disable:
		lda  #2
		sta  debug1
		lda  #%00000011			; no parity, no echo, no XMIT-IRQ,
								;  no RECV-IRQ, RTS off, DTR low
		sta  swift_command
		lda  smode
		and  #$ff-$40
		sta  smode
		plp
		rts
				
nmi_handler:
		lda  swift_status
		and  #%00001000			; test bit 3
		beq  nmi_ret			; skip, if nothing has been received
		lda  swift_io			; read byte from swiftlink
		ldx  r_wrpointer
		sta  r_buffer,x			; store byte in receive buffer
		inx
		stx  r_wrpointer
		inx
		cpx  r_rdpointer
		beq	 +					; buffer is full

nmi_ret:				
		pla						; restore memory-configuration
		sta  1
		pla						; restore register and return
		tay
		pla
		tax
		pla
		rti


	+	lda  #%00000011			; no parity, no echo, no XMIT-IRQ,
								;  no RECV-IRQ, RTS off, DTR low
		sta  swift_command
		lda  #2
		sta  debug1
		lda  smode
		ora  #$80
		sta  smode
		jmp  nmi_ret
		
rs232_getc:
		sei
		stx  tmpzp+1
		ldx  r_rdpointer
		cpx  r_wrpointer
		beq  null				; no char available
		lda  smode
		cmp  #%11000000			; NMI allowed, but currently disabled
		bne  +
		;; reactivate NMI
		lda  #5
		sta  debug1
		lda  #%00001001			; no parity, no echo, no XMIT-IRQ,
								;  RECV-IRQ enabled, RTS on, DTR low
		sta  swift_command
		
	+	lda  smode
		bpl  +
		and  #$ff-$80			; buffer is not full anymore
		sta  smode
		
	+	lda  r_buffer,x
		sta  tmpzp
		inx
		stx  r_rdpointer
		ldx  tmpzp+1
		lda  tmpzp
		cli
		rts

null:	ldx  tmpzp+1
		cli
		lda  #0
		rts

		
	-	lda  tmpzp
		cli
		
rs232_putc:
		sei
		sta  tmpzp
		lda  swift_status
		and  #%00010000			; test bit 4
		beq  -					; wait until current byte is sent
		lda  tmpzp
		sta  swift_io			; send byte
		cli
		rts

		;;
		;;
		
micro_term:		
		jsr  rs232_init
		bcs  pr_error			; skip with error

		ldx  #0
	-	lda  term_txt,x
		beq  term_loop
		jsr  printk
		inx
		bne  -

term_loop:		
		jsr  rs232_getc
		beq  +
		jsr  printk
		jmp  term_loop
	+	jsr  keyb_getc
		beq  term_loop
		jsr  rs232_putc
		jmp  term_loop

pr_error:
		ldx  #0
	-	lda  term_err_txt,x
		beq  term_loop
		jsr  printk
		inx
		bne  -
		rts

smode:	        .buf 1
r_wrpointer:	.buf 1
r_rdpointer:	.buf 1
r_buffer:		.buf 256

term_txt:
		.asc "mICRO TERMINAL v0.1 RUNNING"
		.byte $0a,$00

term_err_txt:
		.asc "ERROR INITIALIZING nmi"
		.byte $0a,$00
