		;; 8 signals 0-7
		;; signal 9 is a pseudo singal that terminates a process
		;;  (make process call suicied with exitcode $7f)

#include "system.h"
#include "kerrors.h"
		
.global sendsignal
.global _raw_sendsignal
.global signal
		
	-	lda  #$7f
		jmp  suicide
		
_kill:	cpx  #32				; kill process X=ipid (exitcode $7f)
		bcs  _err_illarg		; valid processes are 0..31
		cpx  lk_ipid
		beq  -					; kill myself is called suicide! 

		lda  lk_tstatus,x
		and  #tstatus_nosig
		bne  +					; kill already in progress ! (just skip it)
		lda  lk_tstatus,x
		ora  #tstatus_nosig
		sta  lk_tstatus,x		; lock signals for this process

		lda  lk_ttsp,x			; set pointer to task's superpage
		sta  tmpzp+1
		lda  #0
		sta  tmpzp
		
		lda  #6
		sta  tsp_stsize,x		; new stacksize is 6 (y,x,a,sr,pcl,pch)
		ldy  #256-6
		lda  #$7f				; exitcode $7f for 'killed in action'
		sta  (tmpzp),y          ; (Y=exitcode)
		iny
		sta  (tmpzp),y          ; (X=exitcode)
		iny
		sta  (tmpzp),y          ; (A=exitcode)
		lda  #50
		iny
		sta  (tmpzp),y          ; (SR=50)
		lda  #<suicide
		iny
		sta  (tmpzp),y          ; (pc=suicide)
		lda  #>suicide
		iny
		sta  (tmpzp),y
		
	+	cli
		lda  #0					; return A=0 (signal sent)
		clc
		rts						; done
     
	-	jmp  catcherr

_err_illarg:
		lda  #lerr_illarg
		jmp  suicerrout
		
		;; send signal
		;; A/Y=PID of destination process
		;; X=number of signal (0..7 or 9 for kill)
		
sendsignal:  
		sei						; send signal to process A/Y=PID, X=signal
		stx  tmpzp+4
		jsr  getipid
		bcs  -

		lda  tmpzp+4			; check if its a valid signal
		cmp  #9
		beq  _kill				; signal 9 is kill !
		cmp  #8
		bcs  _err_illarg		; only signals 0-7 are valid
          
		; raw-sendsignal-routine (tmpzp+4=signum, X=ipid)

		jsr  _raw_sendsignal
		cli
		rts
		
_raw_sendsignal:
		lda  lk_tstatus,x       
		and  #tstatus_nosig
		bne  skip_signal		; no signals ?, then don't send signal
		
		lda  #0					; get pointer to stack
		sta  tmpzp
		lda  lk_ttsp,x
		sta  tmpzp+1

		ldy  #tsp_stsize
		lda  (tmpzp),y			; enough space left in stack ?
		clc
		adc  #16				; need 3 bytes but skip if less than 16 left
		ldy  #tsp_zpsize
		adc  (tmpzp),y
		adc  #tsp_swap
		bcs  skip_signal		; no, then skip

		lda  tmpzp+4			; get pointer to signal vector
		asl  a
		adc  #tsp_signal_vec+1
		tay
		lda  (tmpzp),y			; read hi-byte of vector
		beq  skip_signal		; signal doesn't get caught (ignore it)

		sta  _sig.hi			; remember highbyte
		dey
		lda  (tmpzp),y
		sta  _sig.lo			; remember lowbyte

		cpx  lk_ipid
		beq  self_sig			; send signal to current process

		;; send signal to foreign process
		ldy  #tsp_stsize
		lda  (tmpzp),y			; enough space left on stack ?
		eor  #$ff
		tay
		iny						; (tmpzp),y points to latest stack element
		lda  (tmpzp),y			; load and emember it (its MEMCONV)
		pha
		iny
		lda  (tmpzp),y          ; load and remember Y
		pha
		lda  #50				; and replace with #50 (new SR, c=0!)
		sta  (tmpzp),y
		iny						; load and remember X
		lda  (tmpzp),y
		pha
		lda  _sig.lo			; and replace with sigaddres-low
		sta  (tmpzp),y
		iny						; load and remember A
		lda  (tmpzp),y
		pha
		lda  _sig.hi			; and replace with sigaddres-high
		sta  (tmpzp),y
		dey
		dey
		dey
		pla						; get A
		sta  (tmpzp),y          ; put A as new stack element
		dey
		pla						; get X
		sta  (tmpzp),y          ; put X as new stack element
		dey
		pla						; get Y
		sta  (tmpzp),y          ; put Y as new stack element
		dey
		pla						; get MEMCONV
		sta  (tmpzp),y          ; put MEMCONV as new stack element
		dey
		tya
		eor  #$ff
		ldy  #tsp_stsize
		sta  (tmpzp),y			; write new stacksize
		lda  #0					; load returncode 0 ('signal sent')
		.byte $2c				; skip next instruction

skip_signal:	
		lda  #255				; load returncode $ff ('signal not sent')
		clc						; return with carry clear
		rts

self_sig:		
		tsx						; send signal to current process
		inx						; adapt return address (+1)
		inc  $100,x
		bne  +
		inx
		inc  $100,x
	+	lda  #50				; push pseudo SR=50 on stack
		pha						; return with c=0, i=0

		sec						; the signal-handler can check the carry-flag
								; if it is 1, the signal was sent by the
								; process itself !       

		lda  #0					; return with A=0 (signal sent)

		_sig.lo equ *+1
		_sig.hi equ *+2
		
		jmp  $0000				; jump to signaladdess directly

	-	jmp  _err_illarg
		
		;; signal
		;; install a signal handler
		;; <- X=signal number 0..7, A/Y=address of handler
		;; (Y=0 de-installs handler)
		;; -> c=error

signal:	cpx  #8
		bcs  -					; only signals 0-7 are valid
		pha
		tya
		pha
		txa
		asl  a
		adc  #tsp_signal_vec+1
		tay
		pla						; hi-byte of handler address
		sei
		sta  (lk_tsp),y
		dey
		pla						; lo-byte of handler address
		sta  (lk_tsp),y
		cli
		rts		
